@@ -1,5 +1,10 @@
+# fields, Tools for spatial data
+# Copyright 2004-2007, Institute for Mathematics Applied Geosciences
+# University Corporation for Atmospheric Research
+# Licensed under the GPL -- www.gpl.org/licenses/gpl.html
 
-Krig.Amatrix<- function (object, x0 = object$x, lambda=NULL)
+
+Krig.Amatrix<- function (object, x0 = object$x, lambda=NULL,...)
 {
 
   if( is.null( lambda)){ lambda<- object$lambda}
@@ -19,7 +24,7 @@ Krig.Amatrix<- function (object, x0 = object$x, lambda=NULL)
     for( k in 1: M){
      ytemp<- rep( 0,M)
      ytemp[k] <- 1
-     out[,k] <- predict(object, x= x0, yM= ytemp, lambda=lambda)
+     out[,k] <- predict(object, x= x0, yM= ytemp, lambda=lambda,...)
     } 
 
     return(out)
