@@ -1,3 +1,8 @@
+# fields, Tools for spatial data
+# Copyright 2004-2007, Institute for Mathematics Applied Geosciences
+# University Corporation for Atmospheric Research
+# Licensed under the GPL -- www.gpl.org/licenses/gpl.html
+
 "Krig.engine.default" <-
 function(out, verbose=FALSE){
 # 
@@ -20,49 +25,79 @@ function(out, verbose=FALSE){
 # the vectors d and c. 
 #
 
-#  W  be the weight matrix for unique locations e.g. diag( out$weightsM)
+#  First outline calculations with equal weights
 #  T the fixed effects regression matrix  T.ij = phi.j(xM.i)
 #  K the covariance matrix for the unique locations
 
 # From the spline literature the solution solves the well known system 
 # of two eqautions:
 
-#    -KW( yM - Td - Kc) + lambda *Kc = 0
-#     -T^t W ( yM-Td -Kc) =0
+#    -K( yM - Td - Kc) + lambda *Kc = 0
+#     -T^t ( yM-Td -Kc) =0
 
 # Divide through by K and substitute, these are equivalent to 
 
-#       -W( yM- Td - Kc) + lambda c = 0
-#        T^t c = 0       
+#  -1-      -( yM- Td - Kc) + lambda c = 0
+#  -2-      T^t c = 0       
 
 
 
 #
-#  A QR decomposition is done for   sqrt(W)T= (Q.1,Q.2)R
+#  A QR decomposition is done for   T= (Q.1,Q.2)R
+#   by definition  Q.2^T T =0
+  
+#  equation  -2- can be thought of as a constraint
+# with  c= Q.2 beta2
+# substitute in  -1-  and multiply through by Q.2^T
+
+#      -Q.2^T yM  + Q.2^T K Q.2 beta2  + lambda beta2 = 0
 #
+#   Solving
+#   beta2 = {Q.2^T K Q.2 beta2  + lambda I )^ {-1} Q.2^T yM
+# 
 #  eigenvalues and eigenvectors found for M= Q.2^T K Q.2 
-#  
-
-
 #
-# QR decomposition for the matrix defining the fixed effects or the 
-# null space. 
-# Note premultiplication by sqrt of weights 
-#   QR   where Q= ( Q_1, Q_2)  Q_1 spans column space of T
+#
+# From -1-  Td = yM - Kc - lambda c
+#      (Q.1^T) Td =  (Q.1^T) ( yM- Kc)    
+#
+#   ( lambda c is zero by constraint)
+#   
+#   so Rd = (Q.1^T) ( yM- Kc)
+# use qr functions to find d. 
+# 
+
+
+        Tmatrix<- do.call(out$null.function.name, 
+                           c(out$null.args, list(x=out$xM,Z=out$ZM))  )
+        if( verbose){
+             cat(" Model Matrix: spatial drift and Z", fill=TRUE)
+             print( Tmatrix)
+        }
+
+# Tmatrix premultiplied by sqrt of wieghts
+        Tmatrix<- out$W2%d*%Tmatrix 
+
+        qr.T <- qr(Tmatrix )
 
-        qr.T <- qr(c(sqrt(out$weightsM)) * out$make.tmatrix(out$xM, out$m))
 #
 #verbose block
         if (verbose) {
-cat( "first 5 rows of qr.T$qr",fill=TRUE)
+            cat( "first 5 rows of qr.T$qr",fill=TRUE)
             print(qr.T$qr[1:5,])
         }
 #
 # find  Q_2 K Q_2^T  where K is the covariance matrix at the knot points
 #
-        tempM <- sqrt(out$weightsM) * t(sqrt(out$weightsM) * 
-            t(do.call(out$cov.function.name, 
-            c(out$args, list(x1 = out$knots, x2 = out$knots)))))
+        
+        tempM<-   t(
+           out$W2 %d*% 
+           do.call(out$cov.function.name, 
+               c(out$args, list(x1 = out$knots, x2 = out$knots)))
+                    )
+
+        tempM <- out$W2 %d*% tempM
+        
         tempM <- qr.yq2(qr.T, tempM)
         tempM <- qr.q2ty(qr.T, tempM)
 
@@ -70,15 +105,14 @@ cat( "first 5 rows of qr.T$qr",fill=TRUE)
     nt <- (qr.T$rank)
 
     if (verbose) {
-        cat("np, nt", np, nt, fill = TRUE)
-}
+        cat("np, nt", np, nt, fill = TRUE)}
 
 #
 # Full set of decompositions for 
 # estimator for nonzero lambda
 
-            temp <- eigen(tempM, symmetric=TRUE)
-            D <- c(rep(0, nt), 1/temp$values)
+            tempM <- eigen(tempM, symmetric=TRUE)
+            D <- c(rep(0, nt), 1/tempM$values)
 #
 # verbose block
             if (verbose) {
@@ -90,18 +124,22 @@ cat( "first 5 rows of qr.T$qr",fill=TRUE)
 # evaluate the solution, GCV, REML  at different lambdas
 #
             G <- matrix(0, ncol = np, nrow = np)
-            G[(nt + 1):np, (nt + 1):np] <- temp$vectors
+        
+            G[(nt + 1):np, (nt + 1):np] <- tempM$vectors
+
             G <- G * matrix(D, ncol = np, nrow = np, byrow = TRUE)
-            u <- c(rep(0, nt), t(temp$vectors) %*% qr.q2ty(qr.T, 
-                        sqrt(out$weightsM) * out$yM))
+
+            u <- c( rep(0, nt),
+                t(tempM$vectors) %*% 
+                    qr.q2ty(qr.T, c(out$W2%d*%out$yM ) ))
 #
 # verbose block
-            if (verbose) {
+            if (verbose){
                 print(u)
                 print(out$pure.ss)
             }
 #
             return( list(u = u, D = D, G = G, qr.T = qr.T, 
-                decomp = "WBW", V = temp$vectors, nt=nt, np=np))
+                decomp = "WBW", V = tempM$vectors, nt=nt, np=np))
 }
 
